@@ -30,6 +30,7 @@ if system == "Darwin":
     configuration_path_prefix = os.path.expanduser("~")
 configuration_file = "{}/apps/public/conf.ini".format(configuration_path_prefix)
 
+
 def getConfig():
     print(configuration_file)
     import configparser
@@ -376,17 +377,39 @@ def weixin_send_text(message, token=None, mobiles=[], is_at_all=False):
                   auth=('Content-Type', 'application/json'))
 
 
-def get_md5_code(content):
+def get_md5_code(arg, is_file=False):
     """
-    :param content: 原始字符串
+    :param is_file: 如果为True，就对文件做md5处理
+    :param arg: 原始字符串，或者文件路径
     :return : 对原始字符串进行MD5加密的字符串
     """
     m = hashlib.md5()
-    b = content.encode(encoding='utf-8')
-    m.update(b)
+    if is_file:
+        img = open(arg, 'rb')
+        m.update(img.read())
+        img.close()
+    else:
+        b = arg.encode(encoding='utf-8')
+        m.update(b)
     return m.hexdigest()
 
 
+def md5(arg, is_file=False):
+    """
+    :param is_file: 如果为True，就对文件做md5处理
+    :param arg: 原始字符串，或者文件路径
+    :return : 对原始字符串进行MD5加密的字符串
+    """
+    m = hashlib.md5()
+    if is_file:
+        img = open(arg, 'rb')
+        m.update(img.read())
+        img.close()
+    else:
+        b = arg.encode(encoding='utf-8')
+        m.update(b)
+    return m.hexdigest()
+
 # def aes(text,apiSecret):
 #     """
 #     AES加密
@@ -425,22 +448,39 @@ def dejson(content):
     tmp_content = re.sub(",([a-zA-Z])", r',"\1', tmp_content)
     return tmp_content
 
+
 # reconnecting mysql , 参考 https://www.kingname.info/2017/04/17/decorate-for-method/
 def pingmysql(original_function):
     def wrapper(self, *args, **kwargs):
         try:
             self.database.ping()
+            if self.columns_dict is None:
+                cursor = self.database.cursor()
+                self.columns_dict = {}
+                sql = "select TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,DATA_TYPE from information_schema.columns where 1 =1 and TABLE_SCHEMA not in ('information_schema','performance_schema','mysql','sys') "
+                cursor.execute(sql)
+                rows = []
+                for row in cursor:
+                    table_schema = row.get("TABLE_SCHEMA")
+                    table_name = row.get("TABLE_NAME")
+                    column_name = row.get("COLUMN_NAME")
+                    k = '{}.{}.{}'.format(table_schema, table_name, column_name)
+                    self.columns_dict[k] = row
             result = original_function(self, *args, **kwargs)
             return result
-        except Exception:
+        except Exception as e:
+            traceback.print_exc()
             return 'an Exception raised.'
+
     return wrapper
 
 
 class Mysql():
 
     def __init__(self, configuraion_file=configuration_file, section="mysql"):
+        self.default_dabase_name = ''
         self.database = None
+        self.columns_dict = None
         try:
             config = configparser.ConfigParser()
             config.read(configuraion_file)
@@ -449,27 +489,37 @@ class Mysql():
             user = config.get(section, "user")
             password = config.get(section, "passwd")
             database_str = config.get(section, "database")
+            self.default_dabase_name = database_str
             charset = config.get(section, "charset")
 
             self.database = pymysql.connect(host=host, port=port, user=user, passwd=password, db=database_str,
                                             charset=charset, autocommit=True,
                                             cursorclass=pymysql.cursors.DictCursor)
 
-            # def ping_mysql_loop():
-            #     while True:
-            #         if self.database:
-            #             self.database.ping()
-            #             t.sleep(10)
-            #
-            # logger.debug("启动定时ping的线程")
-            # Thread(target=ping_mysql_loop).start()
-
         except ImportError:
             print("Error: configparser or pymysql module not exists")
         except Exception as e:
             logger.error(e)
             traceback.print_exc()
 
+    def is_legal_value(self, table_name, column_name, val):
+        if "." not in table_name:
+            table_name = "{}.{}".format(self.default_dabase_name, table_name)
+        col_name = "{}.{}".format(table_name, column_name)
+        if self.columns_dict is None:
+            return True
+        row = self.columns_dict.get(col_name)
+        data_type = row.get("DATA_TYPE")
+        if data_type in ['int', 'tinyint', 'bigint']:
+            if len(val.strip()) == 0:
+                logger.warning("{} {} is not a valid {}".format(column_name, val, data_type))
+                return False
+            elif not str(val).isnumeric():
+                logger.error("{} {} is not a wrong {}".format(column_name, val, data_type))
+                raise Exception("{} {} is not a wrong {}".format(column_name, val, data_type))
+                return False
+        return True
+
     @pingmysql
     def query(self, sql, argumentTuple=(), timestamp2str=True):
         cursor = self.database.cursor()
@@ -508,7 +558,7 @@ class Mysql():
         id = ""
         for key in dic.keys():
             val = dic[key]
-            if val is not None:
+            if self.is_legal_value(tableName, key, val):
                 cols.append(key)
                 placeholders.append("%s")
                 vals.append(val)
@@ -526,12 +576,24 @@ class Mysql():
 
     @pingmysql
     def update_by_id(self, tableName, id, dic, idFieldName="id", commit=True):
+        """
+        if want to change one field value to NULL, you should put {'fieldname':None} to dic.
+
+        :param tableName:
+        :param id:
+        :param dic:
+        :param idFieldName:
+        :param commit:
+        :return:
+        """
         cursor = self.database.cursor()
         vals = []
         placeholders = []
         for key in dic.keys():
             val = dic[key]
-            if val is not None:
+            if val is None:
+                placeholders.append("{} = null ".format(key))
+            elif self.is_legal_value(tableName, key, val):
                 placeholders.append("{} = %s ".format(key))
                 vals.append(val)
         setting = " , ".join(placeholders)
@@ -610,4 +672,4 @@ class Mysql():
         for start in range(0, len(values_array), batch_size):
             logger.debug(values_array[start:start + batch_size])
             cursor.executemany(ql, values_array[start:start + batch_size])
-            self.database.commit()
\ No newline at end of file
+            self.database.commit()
